<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects;

use InvalidArgumentException;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\CountryAndLanguage\{Continent, ISO3166Alpha2, ISO3166Alpha3, ISO3166Numeric, Name};
use Locr\WordPress\Plugin\LocrMaps\TypedArray;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the options.
 *
 * @property-read Continent $continent The continent.
 * @property-read ISO3166Alpha2 $iso3166alpha2 The ISO 3166-1 alpha-2 code.
 * @property-read ISO3166Alpha3 $iso3166alpha3 The ISO 3166-1 alpha-3 code.
 * @property-read ISO3166Numeric $iso3166numeric The ISO 3166-1 numeric code.
 * @property-read TypedArray $localized_names The localized names.
 * @property-read Name $name The name.
 */
final class CountryAndLanguage {
	/**
	 * The constructor.
	 *
	 * @param Name           $name The name.
	 * @param TypedArray     $localized_names The localized names.
	 * @param ISO3166Alpha2  $iso3166alpha2 The ISO 3166-1 alpha-2 code.
	 * @param ISO3166Alpha3  $iso3166alpha3 The ISO 3166-1 alpha-3 code.
	 * @param ISO3166Numeric $iso3166numeric The ISO 3166-1 numeric code.
	 * @param Continent      $continent The continent.
	 */
	public function __construct(
		private Name $name,
		private TypedArray $localized_names,
		private ISO3166Alpha2 $iso3166alpha2,
		private ISO3166Alpha3 $iso3166alpha3,
		private ISO3166Numeric $iso3166numeric,
		private Continent $continent
	) {
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'continent' => $this->continent,
			'iso3166alpha2' => $this->iso3166alpha2,
			'iso3166alpha3' => $this->iso3166alpha3,
			'iso3166numeric' => $this->iso3166numeric,
			'localized_names' => $this->localized_names,
			'name' => $this->name,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * Processes the data from an array.
	 *
	 * @param array<mixed> $data The data.
	 * @throws \Exception If the data is invalid.
	 */
	public static function from_array( array $data ): self {
		if ( ! isset( $data['name'] ) ) {
			throw new \Exception( 'CountryAndLanguage: The "name" is missing from the data.' );
		}
		if ( ! isset( $data['ISO3166-1:alpha2'] ) ) {
			throw new \Exception( 'CountryAndLanguage: The "ISO3166-1:alpha2" is missing from the data.' );
		}
		if ( ! isset( $data['ISO3166-1:alpha3'] ) ) {
			throw new \Exception( 'CountryAndLanguage: The "ISO3166-1:alpha3" is missing from the data.' );
		}
		if ( ! isset( $data['ISO3166-1:numeric'] ) ) {
			throw new \Exception( 'CountryAndLanguage: The "ISO3166-1:numeric" is missing from the data.' );
		}
		if ( ! isset( $data['continent'] ) ) {
			throw new \Exception( 'CountryAndLanguage: The "continent" is missing from the data.' );
		}

		$localized_names = TypedArray::for_type( Name::class );
		foreach ( $data as $key => $value ) {
			if ( preg_match( '/^name:(?P<language>[a-z]{2})$/', $key, $matches ) ) {
				$localized_names[ $matches['language'] ] = new Name( $value );
			}
		}

		return new self(
			name: new Name( $data['name'] ),
			localized_names: $localized_names,
			iso3166alpha2: new ISO3166Alpha2( $data['ISO3166-1:alpha2'] ),
			iso3166alpha3: new ISO3166Alpha3( $data['ISO3166-1:alpha3'] ),
			iso3166numeric: new ISO3166Numeric( $data['ISO3166-1:numeric'] ),
			continent: new Continent( $data['continent'] )
		);
	}

	/**
	 * Tries to get a localized name.
	 *
	 * @param string $locale The locale.
	 */
	public function try_get_localized_name( string $locale ): Name {
		if ( isset( $this->localized_names[ $locale ] ) && $this->localized_names[ $locale ] instanceof Name ) {
			return $this->localized_names[ $locale ];
		}

		if ( preg_match( '/^(?P<language_code>[a-z]{2})[_-](?P<country_code>[A-Z]{2})$/', $locale, $matches ) ) {
			if ( isset( $this->localized_names[ $matches['language_code'] ] ) && $this->localized_names[ $matches['language_code'] ] instanceof Name ) {
				return $this->localized_names[ $matches['language_code'] ];
			}
		}

		return $this->name;
	}
}
